"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[5164],{25164:function(o){o.exports=JSON.parse('{"base":"vs","inherit":true,"rules":[{"background":"FFFFFF","token":""},{"foreground":"8c868f","token":"comment"},{"foreground":"3b5bb5","token":"constant"},{"foreground":"3b5bb5","token":"entity"},{"foreground":"d62a28","token":"text.tex.latex entity"},{"foreground":"ff7800","token":"keyword"},{"foreground":"ff7800","token":"storage"},{"foreground":"409b1c","token":"string"},{"foreground":"409b1c","token":"meta.verbatim"},{"foreground":"3b5bb5","token":"support"},{"foreground":"990000","fontStyle":"italic","token":"invalid.deprecated"},{"foreground":"f8f8f8","background":"9d1e15","token":"invalid.illegal"},{"foreground":"3b5bb5","fontStyle":"italic","token":"entity.other.inherited-class"},{"foreground":"671ebb","token":"string constant.other.placeholder"},{"foreground":"3e4558","token":"meta.function-call.py"},{"foreground":"3a4a64","token":"meta.tag"},{"foreground":"3a4a64","token":"meta.tag entity"},{"foreground":"7f90aa","token":"keyword.type.variant"},{"foreground":"000000","token":"source.ocaml keyword.operator"},{"foreground":"3b5bb5","token":"source.ocaml keyword.operator.symbol.infix"},{"foreground":"3b5bb5","token":"source.ocaml keyword.operator.symbol.prefix"},{"fontStyle":"underline","token":"source.ocaml keyword.operator.symbol.infix.floating-point"},{"fontStyle":"underline","token":"source.ocaml keyword.operator.symbol.prefix.floating-point"},{"fontStyle":"underline","token":"source.ocaml constant.numeric.floating-point"}],"colors":{"editor.foreground":"#000000","editor.background":"#FFFFFF","editor.selectionBackground":"#E3FC8D","editor.lineHighlightBackground":"#EFFCA68F","editorCursor.foreground":"#7C7C7C","editorWhitespace.foreground":"#B6B6B6"}}')}}]);