"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[3467],{33467:function(e){e.exports=JSON.parse('{"base":"vs-dark","inherit":true,"rules":[{"background":"404040","token":""},{"foreground":"709070","fontStyle":"italic","token":"comment"},{"fontStyle":"bold","token":"keyword.other.directive"},{"fontStyle":"underline","token":"keyword.other.directive.line-number"},{"foreground":"ff8080","token":"constant.character"},{"foreground":"ff2020","token":"string"},{"foreground":"22c0ff","token":"constant.numeric"},{"fontStyle":"underline","token":"constant.numeric.floating-point"},{"foreground":"ffffa0","token":"keyword"},{"foreground":"ff8000","fontStyle":"bold","token":"entity.name.module"},{"foreground":"ff8000","fontStyle":"bold","token":"support.other.module"},{"foreground":"ffffa0","token":"keyword.operator"},{"fontStyle":"underline","token":"source.ocaml keyword.operator.symbol.infix.floating-point"},{"fontStyle":"underline","token":"source.ocaml keyword.operator.symbol.prefix.floating-point"},{"foreground":"6080ff","token":"storage.type"},{"foreground":"4080a0","token":"entity.name.class.variant"},{"foreground":"f09040","token":"entity.name.type"},{"foreground":"ffcc66","fontStyle":"bold","token":"entity.name.function"},{"foreground":"ffe000","token":"storage.type.user-defined"},{"foreground":"f4a020","token":"entity.name.type.class.type"}],"colors":{"editor.foreground":"#DEDEDE","editor.background":"#404040","editor.selectionBackground":"#A0A0C0","editor.lineHighlightBackground":"#A0804026","editorCursor.foreground":"#FFFF66","editorWhitespace.foreground":"#A8A8A8"}}')}}]);