(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[5405],{96510:function(e,n,r){"use strict";r.d(n,{nU:function(){return s},pr:function(){return a},qQ:function(){return d},wD:function(){return f}});var t=r(77555),o=r(82394),c=r(12757);function i(e,n){var r=Object.keys(e);if(Object.getOwnPropertySymbols){var t=Object.getOwnPropertySymbols(e);n&&(t=t.filter((function(n){return Object.getOwnPropertyDescriptor(e,n).enumerable}))),r.push.apply(r,t)}return r}function u(e){for(var n=1;n<arguments.length;n++){var r=null!=arguments[n]?arguments[n]:{};n%2?i(Object(r),!0).forEach((function(n){(0,o.Z)(e,n,r[n])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(r)):i(Object(r)).forEach((function(n){Object.defineProperty(e,n,Object.getOwnPropertyDescriptor(r,n))}))}return e}function s(e){var n=e.error,r=n.code,t=n.errors,o=n.message,i=n.type,s=[];return s=t?null!==t&&void 0!==t&&t.__all__?null===t||void 0===t?void 0:t.__all__:Array.isArray(t)?t:Object.entries(t).reduce((function(e,n){var r=(0,c.Z)(n,2),t=r[0],o=r[1];return e.concat("".concat(t,": ").concat(o[0]))}),[]):n.messages,u(u({},n),{},{code:r,errors:t,message:o,messages:s,type:i})}function a(e){return e.data?Promise.resolve(e.data):e.json?e.json():void 0}function l(e){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=s(e),o=r.code,c=r.errors,i=r.exception,u=r.message,a=r.messages,l=[];if(u)l.push.apply(l,(0,t.Z)(u.split("\n")));else if((null===a||void 0===a?void 0:a.length)>=1)l.push.apply(l,(0,t.Z)(a));else{var v=n.errorMessage||(null===a||void 0===a?void 0:a[0])||c;v&&l.push(v)}return i&&l.push(i),{code:o,messages:l}}function v(e){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=n.acceptErrorStatuses,t=void 0===r?[]:r,o=n.callback,c=n.onErrorCallback,i=(n.successMessage,e.error);if(i&&!t.includes(null===i||void 0===i?void 0:i.code)){var u=l(e);return null===c||void 0===c||c(e,u),u}return null===o||void 0===o?void 0:o(e)}function d(e){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=e.errors,t=e.message,o=e.response,c=u({error:{code:null===o||void 0===o?void 0:o.status,messages:[r||t]}},null===o||void 0===o?void 0:o.data),i=n.callback;return null===i||void 0===i||i(c),l(c,n)}function f(e){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return e.status?a(e).then((function(e){return v(e,n)})):v(e,n)}},63878:function(e,n,r){"use strict";r.r(n);var t=r(34376),o=r.n(t),c=r(82684),i=r(41374);n.default=function(){var e=(0,t.useRouter)(),n=e.query,r=e.asPath.split("?")[0],u=i.ZP.pipelines.list().data,s=null===u||void 0===u?void 0:u.pipelines,a="";r&&"/"!==r?a=r:(null===s||void 0===s?void 0:s.length)>=1&&(a+="/pipelines/".concat(s[0])),(0,c.useEffect)((function(){(null===s||void 0===s?void 0:s.length)>=1&&o().push({pathname:a,query:n})}),[a])}},59e3:function(e,n,r){"use strict";r.d(n,{iV:function(){return o},uM:function(){return c}});var t=r(12757);r(34376);function o(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:null,n={},r=e;if(e||(r=window.location.search),r){var o=new URLSearchParams(r.split("?").slice(1).join(""));Array.from(o.keys()).forEach((function(e){var r=o.getAll(e);if(1===r.length){var c=(0,t.Z)(r,1);n[e]=c[0]}else n[e]=r}))}return n}function c(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return Object.entries(e).reduce((function(e,n){var r=(0,t.Z)(n,2),o=r[0],c=r[1];if(Array.isArray(c)){var i=o;return i.match(/\[\]/)||(i="".concat(i,"[]")),e.concat(c.map((function(e){return"".concat(i,"=").concat(encodeURIComponent(e))})))}return e.concat("".concat(o,"=").concat(encodeURIComponent(c)))}),[]).join("&")}},48312:function(e,n,r){(window.__NEXT_P=window.__NEXT_P||[]).push(["/",function(){return r(63878)}])}},function(e){e.O(0,[4259,1374,9774,2888,179],(function(){return n=48312,e(e.s=n);var n}));var n=e.O();_N_E=n}]);