self.__BUILD_MANIFEST=function(s,e,a,t,c,d,i,n,_){return{__rewrites:{beforeFiles:[],afterFiles:[{source:"/CLOUD_NOTEBOOK_BASE_PATH_PLACEHOLDER_/datasets",destination:e},{source:"/CLOUD_NOTEBOOK_BASE_PATH_PLACEHOLDER_",destination:e}],fallback:[]},"/":[s,c,"static/chunks/pages/index-080f7578ee6e7d6e.js"],"/_error":["static/chunks/pages/_error-235304e5badb19eb.js"],"/datasets":[s,a,t,d,"static/chunks/pages/datasets-84bdd76076659514.js"],"/datasets/[...slug]":[i,"static/chunks/8a28b14e-b845f39223877713.js",s,a,n,"static/chunks/1921-fea0f316eb9b5aa2.js",t,_,d,"static/chunks/pages/datasets/[...slug]-c15ce46e9fa4b86b.js"],"/pipelines/[...slug]":[i,s,a,n,"static/chunks/5506-2c13e214763de4cc.js",t,_,c,"static/chunks/pages/pipelines/[...slug]-48291085668e956a.js"],sortedPages:["/","/_app","/_error",e,"/datasets/[...slug]","/pipelines/[...slug]"]}}("static/chunks/4259-f723eee7c5c19f8e.js","/datasets","static/chunks/240-156e8ebf09bd27ad.js","static/chunks/7673-b70b3ad9f212b4d3.js","static/chunks/1374-8524d48985929476.js","static/chunks/3887-5c8404eec6532f49.js","static/chunks/29107295-989a0767a635d9d5.js","static/chunks/1125-861e646db7510d96.js","static/chunks/8952-88d806702dc835d9.js"),self.__BUILD_MANIFEST_CB&&self.__BUILD_MANIFEST_CB();