"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[8018],{68018:function(e){e.exports=JSON.parse('{"base":"vs-dark","inherit":true,"rules":[{"background":"000000","token":""},{"foreground":"aeaeae","fontStyle":"italic","token":"comment"},{"foreground":"3387cc","token":"constant"},{"foreground":"89bdff","token":"entity"},{"foreground":"e28964","token":"keyword"},{"foreground":"99cf50","token":"storage"},{"foreground":"65b042","token":"string"},{"foreground":"9b859d","token":"support"},{"foreground":"3e87e3","token":"variable"},{"foreground":"fd5ff1","fontStyle":"italic underline","token":"invalid.deprecated"},{"foreground":"fd5ff1","background":"562d56bf","token":"invalid.illegal"},{"background":"b1b3ba08","token":"text source"},{"foreground":"9b5c2e","fontStyle":"italic","token":"entity.other.inherited-class"},{"foreground":"daefa3","token":"string.quoted source"},{"foreground":"ddf2a4","token":"string constant"},{"foreground":"e9c062","token":"string.regexp"},{"foreground":"cf7d34","token":"string.regexp constant.character.escape"},{"foreground":"cf7d34","token":"string.regexp source.ruby.embedded"},{"foreground":"cf7d34","token":"string.regexp string.regexp.arbitrary-repitition"},{"foreground":"8a9a95","token":"string variable"},{"foreground":"dad085","token":"support.function"},{"foreground":"cf6a4c","token":"support.constant"},{"foreground":"8996a8","token":"meta.preprocessor.c"},{"foreground":"afc4db","token":"meta.preprocessor.c keyword"},{"fontStyle":"underline","token":"entity.name.type"},{"foreground":"676767","fontStyle":"italic","token":"meta.cast"},{"foreground":"494949","token":"meta.sgml.html meta.doctype"},{"foreground":"494949","token":"meta.sgml.html meta.doctype entity"},{"foreground":"494949","token":"meta.sgml.html meta.doctype string"},{"foreground":"494949","token":"meta.xml-processing"},{"foreground":"494949","token":"meta.xml-processing entity"},{"foreground":"494949","token":"meta.xml-processing string"},{"foreground":"89bdff","token":"meta.tag"},{"foreground":"89bdff","token":"meta.tag entity"},{"foreground":"e0c589","token":"source entity.name.tag"},{"foreground":"e0c589","token":"source entity.other.attribute-name"},{"foreground":"e0c589","token":"meta.tag.inline"},{"foreground":"e0c589","token":"meta.tag.inline entity"},{"foreground":"e18964","token":"entity.name.tag.namespace"},{"foreground":"e18964","token":"entity.other.attribute-name.namespace"},{"foreground":"cda869","token":"meta.selector.css entity.name.tag"},{"foreground":"8f9d6a","token":"meta.selector.css entity.other.attribute-name.tag.pseudo-class"},{"foreground":"8b98ab","token":"meta.selector.css entity.other.attribute-name.id"},{"foreground":"9b703f","token":"meta.selector.css entity.other.attribute-name.class"},{"foreground":"c5af75","token":"support.type.property-name.css"},{"foreground":"f9ee98","token":"meta.property-group support.constant.property-value.css"},{"foreground":"f9ee98","token":"meta.property-value support.constant.property-value.css"},{"foreground":"8693a5","token":"meta.preprocessor.at-rule keyword.control.at-rule"},{"foreground":"dd7b3b","token":"meta.property-value support.constant.named-color.css"},{"foreground":"dd7b3b","token":"meta.property-value constant"},{"foreground":"8f9d6a","token":"meta.constructor.argument.css"},{"foreground":"f8f8f8","background":"0e2231","fontStyle":"italic","token":"meta.diff"},{"foreground":"f8f8f8","background":"0e2231","fontStyle":"italic","token":"meta.diff.header"},{"foreground":"f8f8f8","background":"420e09","token":"markup.deleted"},{"foreground":"f8f8f8","background":"4a410d","token":"markup.changed"},{"foreground":"f8f8f8","background":"253b22","token":"markup.inserted"},{"foreground":"e9c062","fontStyle":"italic","token":"markup.italic"},{"foreground":"e9c062","fontStyle":"bold","token":"markup.bold"},{"foreground":"e18964","fontStyle":"underline","token":"markup.underline"},{"foreground":"e1d4b9","background":"fee09c12","fontStyle":"italic","token":"markup.quote"},{"foreground":"fedcc5","background":"632d04","token":"markup.heading"},{"foreground":"fedcc5","background":"632d04","token":"markup.heading entity"},{"foreground":"e1d4b9","token":"markup.list"},{"foreground":"578bb3","background":"b1b3ba08","token":"markup.raw"},{"foreground":"f67b37","fontStyle":"italic","token":"markup comment"},{"foreground":"60a633","background":"242424","token":"meta.separator"},{"background":"eeeeee29","token":"meta.line.entry.logfile"},{"background":"eeeeee29","token":"meta.line.exit.logfile"},{"background":"751012","token":"meta.line.error.logfile"}],"colors":{"editor.foreground":"#F8F8F8","editor.background":"#000000","editor.selectionBackground":"#DDF0FF33","editor.lineHighlightBackground":"#FFFFFF0D","editorCursor.foreground":"#A7A7A7","editorWhitespace.foreground":"#CAE2FB3D"}}')}}]);